#------------------------------------------------------------------
# $Copyright: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Header: /project/isv-agent/VProAgents/SunJESLDAP/SunJESLDAP.pm,v 1.32 2011/10/17 12:18:07 ssrivast Exp $
# $Author: ssrivast $
# $Date: 2011/10/17 12:18:07 $
# $Revision: 1.32 $
#
# Provides:                            SunJESLDAP
# Author:                              Anshuman Atre
# Creation Date:                       28 Nov, 2005
# Modification History:
# Known bugs/caveats:
#------------------------------------------------------------------

#------------
# Be strict..
#------------
use strict;

#----------------------
# Declare the package..
#----------------------
package SunJESLDAP;

#---------------------------------
# Include standard PERL packages..
#---------------------------------
use Fcntl;
use Socket;
use POSIX qw ( tmpnam );

#-----------------------------------------
# Define module-vendor specific commands..
#-----------------------------------------
use ag_i18n_inc;

#-------------------------------------
# Include required framework modules..
#-------------------------------------
use Sys;            # support generalized UNIX commands
use Proc;           # For calls to the process routines
use Comms;          # For calls to socket(3C) routines
use Version;        # Version display generator
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses

#----------------------------------------------------
# Hard code if you still didn't get it from the env..
#----------------------------------------------------
my ( $CLUS_HOME, $sPrefix ) = ();

#------------------------
# Setup VCS/VCSone home..
#------------------------
if ( $ENV { CLUSTER_HOME } ) { # VCS 5.x/VCSOne..
  $CLUS_HOME ||= $ENV { CLUSTER_HOME } ||= "/opt/VRTSvcsone";
}
else { # VCS 4.x..
  $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
}

( $sPrefix ) = $CLUS_HOME =~ /.*(vad|vcs(|one))$/;

my $sCmdVcsDecrypt = $HADECRYPT || "$CLUS_HOME/sbin/${sPrefix}decrypt";
my $sCmdHares  = $HARES || "$CLUS_HOME/bin/hares";
my $sCmdHatype = $HATYPE || "$CLUS_HOME/bin/hatype";

#-----------------------------------------------------------------------
# Add our version number to the version hash and check if versions are
# requested..
#-----------------------------------------------------------------------
my $sVer = "\$Id: SunJESLDAP.pm,v 1.32 2011/10/17 12:18:07 ssrivast Exp $::ETVHeader";
Version::TellVersion ( $sVer );

#-------------------------------
# Define list to use in online..
#-------------------------------
my @lsRunningProcs = (); # Global list..

#--------------------------------
# Define SunJESLDAP subroutines..
#--------------------------------

=pod

 -------------------------------------------------------------------------
 Function :  bool ArgsValid ( IN String EntryPointName,
                              IN String ResourceName,
                              IN String InstanceRoot,
                              IN String LDAPHost,
                              IN Integer LDAPPort,
                              IN String MonitorProgram,
                              IN String SSLPort,
                              IN String SSLDbPasswd,
                              IN Integer SecondLevelMonitor );

 Description : Validates the ArgList passed to the entry point.

 Parameters :

 $sEntryPointName               Entry pt name
 $sResourceName                 Name of this VCS resource
 $localInstanceRoot             Absolute path to instance of Directory Server
 $localLDAPHost                 Hostname/IPv4 address of virtual host for Directory Server
 $localLDAPPort                 The port at which the Directory Server listens
 $localMonitorProgram           Full path to external monitor program
 $localSSLPort                  SSL Port on which the Directory Server listens
 $localSSLDbPasswd              Encrypted password for trusted database using vcsencrypt
 $localSecondLevelMonitor       Cycle count to run in-depth monitoring
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ArgsValid {
  #----------------------------------------
  # Set up local vars from passed in ones..
  #----------------------------------------
  my ( $sEntryPointName,
       $sResourceName,
       $localInstanceRoot,
       $localLDAPHost,
       $localLDAPPort,
       $localMonitorProgram,
       $localSSLPort,
       $localSSLDbPasswd,
       $localSecondLevelMonitor ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20001, "Subroutine <%s> called with:
  EntryPointName           [%s]
  ResourceName             [%s]
  InstanceRoot             [%s]
  LDAPHost                 [%s]
  LDAPPort                 [%s]
  MonitorProgram           [%s]
  SSLPort                  [%s]
  SSLDbPasswd              [%s]
  SecondLevelMonitor       [%s]",

  $sMyName,
  $sEntryPointName, $sResourceName,  $localInstanceRoot,  $localLDAPHost,
  $localLDAPPort, $localMonitorProgram, $localSSLPort, $localSSLDbPasswd,
  $localSecondLevelMonitor );

  #-------------------------------------
  # Set return code to hopeful success..
  #-------------------------------------
  my $iReturnCode = 1;

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my ( @lsMissing, %hsArgList ) = ();
  my @lsArgs =
  qw ( EntryPointName  ResourceName   InstanceRoot   LDAPHost   LDAPPort
       MonitorProgram SSLPort SSLDbPasswd SecondLevelMonitor );

  @hsArgList { @lsArgs } = @_; # Assign..

  #----------------------------------------
  # Reqd. attributes based on entry point..
  #----------------------------------------
  my @lsOnlineArgs  = qw ( InstanceRoot LDAPHost LDAPPort );
  my @lsOfflineArgs = qw ( InstanceRoot );
  my @lsMonitorArgs = qw ( InstanceRoot );
  my @lsCleanArgs   = qw ( InstanceRoot );

  my %hlsNeeded = ( 'online'  => \@lsOnlineArgs,
                    'offline' => \@lsOfflineArgs,
                    'monitor' => \@lsMonitorArgs,
                    'clean'   => \@lsCleanArgs );

  my @lsReqdArgs = @{$hlsNeeded { $sEntryPointName }};

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20002,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return, if failed..

  #------------------------------------------
  # Force SLM if enabled, for next monitor..
  #------------------------------------------
  if ( $sEntryPointName ne 'monitor' ) {
    VCSagentFW::RunSecondLevelMonitor ( 1 ); # Delete..
  }

  #----------------------------------------
  # Return, if failed or monitor or clean..
  #----------------------------------------
  if ( ! $iReturnCode                  ||
         $sEntryPointName eq 'monitor' ||
         $sEntryPointName eq 'clean' ) {
    return $iReturnCode;
  }

  #-------------------------------
  # Individual validation starts..
  #-------------------------------

  #-------------------------
  # Instance root must exist..
  #-------------------------
  if ( ! -d $localInstanceRoot ) {
    &logError ( 20003, "<%s> InstanceRoot directory [%s] does not exist",
    $sMyName, $localInstanceRoot );
    $iReturnCode = 0; # Failure..
  }

  if ( $sEntryPointName eq 'online' ) {
    #--------------------
    # Validate hostname..
    #--------------------
    if ( ! inet_aton ( $localLDAPHost ) ) {
      &logError ( 20004, "<%s> Could not resolve LDAPHost [%s]",
      $sMyName, $localLDAPHost );
      $iReturnCode = 0; # Failure..
    }

    #------------
    # LDAP Port..
    #------------
    if ( $localSecondLevelMonitor && ! $localLDAPPort ) {
      &logError ( 20005,
      "<%s> LDAPPort [%s] has to be greater than 0 for SLM [%s]",
      $sMyName, $localLDAPPort, $localSecondLevelMonitor );
      $iReturnCode = 0; # Failure..
    }

    #--------------------------------------
    # MonitorProgram has to be executable..
    #--------------------------------------
    if ( $localMonitorProgram =~ /(\S+)/ &&     # Defined..
         ! -x $1 ) {                            # And not executable..
      &logError ( 20006,
      "<%s> MonitorProgram [%s] not executable", $sMyName, $1 );
      $iReturnCode = 0; # Failure..
    }
  }

  $iReturnCode ||
  &logWarn ( 20007,
  "<%s> Some of the attributes could not be validated. Returning [%s]",
  $sMyName, $iReturnCode );

  return $iReturnCode ;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Start ( IN String InstanceRoot,
                         IN String SSLDbPasswd );

 Description : Start logic needed for online entry pt

 Parameters :

 $sInstanceRoot     Absolute path to instance of Directory Server
 $sSSLDbPasswd      Encrypted password for trusted database using vcsencrypt
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Start {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot,
       $sSSLDbPasswd ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20008, "Subroutine <%s> called with:
  InstanceRoot             [%s]
  SSLDbPasswd              [%s]",

  $sMyName, $sInstanceRoot, $sSSLDbPasswd );
  
  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #-----------------------------
  # Initialise local variables..
  #-----------------------------
  my ( $iTillWhen, $iTimeout, @liDummyPids ) = ();
  my $iSleepTime = 10; # Secs..

  #--------------------------------
  # Kill any process if remaining..
  #--------------------------------
  if ( SunJESLDAP::KillLDAPProcs ( $sInstanceRoot ) ) {
    &logTrace ( 20009,
    "<%s> Confirmed no persistent processes exist before startup",
    $sMyName );
  }
  else {
    &logError ( 20010,
    "<%s> Persistent processes exist for Directory Server", $sMyName );
  }

  #----------------------------------
  # Check if SSLDbPasswd is defined..
  #----------------------------------
  if ( $sSSLDbPasswd =~ /\S/ ) { # Not null..
    &logTrace ( 20011,
    "<%s> Starting server using crypted protected database passwd [%s]",
    $sMyName, $sSSLDbPasswd );
  }
  else {
    &logTrace ( 20012, "<%s> Starting server without protected database passwd",
    $sMyName );
  }

  #-------------------------
  # Start the LDAP server..
  #-------------------------
  if ( SunJESLDAP::ExecLDAPCmd ( $sInstanceRoot, $sSSLDbPasswd ) ) {
    &logTrace ( 20013, "<%s> Started the Directory Server successfully!",
    $sMyName );

    #--------------------
    # Delay entry point..
    #--------------------
    $iTimeout = SunJESLDAP::GetEngineTimeout ( 1 ); # With buffer..

    while ( ( $iTillWhen = $^T + $iTimeout - time )  > 10 ) {

      if ( VCSDef_MonitorExit::ONLINE ==
           SunJESLDAP::GetProcState ( $sInstanceRoot, \@liDummyPids, 1 ) ) {
        &logInfo ( 20014,
        "<%s> Confirmed startup of Directory Server with PIDs [%s]",
        $sMyName, join ( ' ', @liDummyPids ) );
        last;
      }

      &logTrace ( 20015, "<%s> Could not confirm startup.
      Delaying exit with [%s] sec in hand", $sMyName, $iTillWhen );

      sleep $iSleepTime; # Pause before re-trying..
    }
  }
  else {
    &logError ( 20016, "<%s> Could NOT start the Directory Server",
    $sMyName );
    $iReturnCode = 0; # Failure..
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool Stop ( IN String InstanceRoot );

 Description : Stop logic needed for offline entry pt

 Parameters :

 $sInstanceRoot           Absolute path to instance of Directory Server
 

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Stop {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20017, "Subroutine <%s> called with:
  InstanceRoot             [%s]",

  $sMyName, $sInstanceRoot );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..

  #-----------------------------
  # Initialise local variables..
  #-----------------------------
  my ( $iTillWhen, $iTimeout, @liDummyPids ) = ();
  my $iSleepTime = 10; # Secs..

  #----------------------------
  # Check if PID files exist..
  #----------------------------
  if ( scalar ( SunJESLDAP::GetPidFiles ( $sInstanceRoot ) ) ) {
   &logTrace ( 20018,
    "<%s> PID file located for the Directory Server", $sMyName );
  }
  else {
    &logWarn ( 20019, "<%s> PID file has problems", $sMyName );
  }

  #-----------------------
  # Stop the LDAP server..
  #-----------------------
  if ( SunJESLDAP::ExecLDAPCmd ( $sInstanceRoot, "" ) ) {
    &logTrace ( 20020, "<%s> Stopped the Directory Server successfully!",
    $sMyName );

    #--------------------
    # Delay entry point..
    #--------------------
    $iTimeout = SunJESLDAP::GetEngineTimeout ( 1 ); # With buffer..

    while ( ( $iTillWhen = $^T + $iTimeout - time ) > 10 ) {

      if ( VCSDef_MonitorExit::OFFLINE ==
           SunJESLDAP::GetProcState ( $sInstanceRoot, \@liDummyPids, 0 ) ) {
        &logInfo ( 20021,
        "<%s> Confirmed shutdown of Directory Server", $sMyName );
        last;
      }

      &logTrace ( 20022,
      "<%s> Could not confirm shutdown. Retrying in [%s] sec.
      Delaying kill with [%s] sec in hand",
      $sMyName, $iSleepTime, $iTillWhen );

      sleep $iSleepTime; # Pause before re-trying..
    }

  }
  else {
    &logError ( 20023, "<%s> Could NOT stop the Directory Server",
    $sMyName );
    $iReturnCode = 0; # Failure..
  }

  #--------------------------------
  # Kill any process if remaining..
  #--------------------------------
  if ( SunJESLDAP::KillLDAPProcs ( $sInstanceRoot ) ) {
    &logTrace ( 20024,
    "<%s> Confirmed no pending processes exist after shutdown",
    $sMyName );
  }
  else {
    &logError ( 20025,
    "<%s> Pending processes exist for Directory Server", $sMyName );
    $iReturnCode = 0; # Failure..
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Kill ( IN String InstanceRoot );

 Description : Forces shutdown of resource needed for clean entry point

 Parameters :

 $sInstanceRoot             Absolute path to instance of Directory Server
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Kill {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20026, "Subroutine <%s> called with:
  InstanceRoot             [%s]",

  $sMyName, $sInstanceRoot );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #---------------------------------------
  # Attempt to stop the Directory Server..
  #---------------------------------------
  if ( SunJESLDAP::ExecLDAPCmd ( $sInstanceRoot, "" ) ) {
    &logTrace ( 20027, "<%s> Stopped the Directory Server successfully!",
    $sMyName );
  }
  else {
    &logError ( 20028, "<%s> Could NOT stop the Directory Server",
    $sMyName );
  }

  #-----------------------------
  # Kill any pending processes..
  #-----------------------------
  if ( SunJESLDAP::KillLDAPProcs ( $sInstanceRoot ) ) {
    &logTrace ( 20029, "<%s> Confirmed no pending processes exist",
    $sMyName );
  }
  else {
    &logWarn ( 20030,
   "<%s> Pending processes exist for Directory Server", $sMyName );
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int GetResourceState ( IN String LDAPHost,
                                   IN String InstanceRoot );

 Description : First level monitor to determine state of resource.

 Parameters :

 $sLDAPHost         Hostname/IPv4 address of virtual host for Directory Server
 $sInstanceRoot     Absolute path to instance of Directory Server
 

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 -------------------------------------------------------------------------

=cut

sub GetResourceState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sLDAPHost,
       $sInstanceRoot ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20031, "Subroutine <%s> called:
  LDAPHost                 [%s]
  InstanceRoot             [%s]",

  $sMyName, $sLDAPHost, $sInstanceRoot, );

  my @liDummyPids = ();
  my $iReturnCode = SunJESLDAP::GetProcState ( $sInstanceRoot,
                                               \@liDummyPids,
                                               1 ); # Try fast FLM..

  if( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    &logTrace ( 20033,
    "<%s> found all required processes",
    $sMyName );
  }
  else {
    &logTrace ( 20034,
    "<%s> did not find all required processes, returning [%s]",
    $sMyName, $iReturnCode );
  }

  @lsRunningProcs = @liDummyPids; # Populate global..

  &logTrace ( 20035,
  "<%s> Filling global array with [%s] elements and Returning [%s]",
  $sMyName, scalar @lsRunningProcs, $iReturnCode );

  return ( $iReturnCode );
}

=pod

 -------------------------------------------------------------------------
 Function : int GetProcState ( IN String InstanceRoot,
                               OUT ListReference OnlinePids );
                               In Boolean UseFFLM );

 Description : Performs the actual first level monitoring

 Parameters :

 $sServerRoot      Absolute path to Instance of LDAP Server
 $rliOnlinePids    PIDs of relevant online processes
 $bUseFFLM         Flag to indicate if fast FLM is to be attempted

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 -------------------------------------------------------------------------

=cut

sub GetProcState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot,
       $rliOnlinePids,
       $bUseFFLM ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20036, "Subroutine <%s> called with:
  InstanceRoot        [%s]
  rliOnlinePids       [%s]
  UseFFLM             [%s]",

  $sMyName, $sInstanceRoot, $rliOnlinePids, $bUseFFLM );

  #----------------------------------------
  # Set Return Code Optimistically to 110..
  #----------------------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #----------------------
  # Initialise Variables..
  #----------------------

  my ( $iProcessCount, $rhlProc, $iPid, $iExitCode,
       $sPsCmd, $sLine, $sLang, $sPidFile, @hProcListHash ) = ();

  my $sPattern = "ns-slapd.*$sInstanceRoot";
  @$rliOnlinePids = (); # Reset..

  #---------------------
  # Attempt to be fast..
  #---------------------
  if ( $bUseFFLM ) {
    if ( 1 == scalar ( ( $sPidFile ) =
         SunJESLDAP::GetPidFiles ( $sInstanceRoot ) ) ) {

      &logTrace ( 20037, "<%s> Located PID file [%s].  Attempting fast FLM.",
      $sMyName, $sPidFile );

      $sPsCmd = "/bin/ps";

      #------------------------------------
      # Search the PID using the PID file..
      #------------------------------------
      if ( open ( PIDFILE, $sPidFile ) ) {
        ( $iPid = <PIDFILE> ) =~ s/\n//; # Read a line.. and chomp..

        if ( $iPid && $iPid =~ /\d+/ ) { # Valid PID..
          &logTrace ( 20038, "<%s> +--- Got PID [%s] from PID file [%s] ---+",
          $sMyName, $iPid, $sPidFile );

          close ( PIDFILE ) ||
          &logWarn ( 20039, "<%s> Could not close PID file [%s] after read: [%s]",
          $sMyName, $sPidFile, $! );

          if ( kill ( 0, $iPid ) ) { # Not a defunct..
            ( $sLine ) = grep { s/\n// || 1 } # Chomp..
                         readpipe ( "$sPsCmd -p $iPid -o args=" );

            #--------------------------
            # Check command execution..
            #--------------------------
            if ( $iExitCode = $? >> 8 ) { # Get errno..
              &logError ( 20040,
              "<%s> ps(1m) command [%s] for PID [%s] failed with exit code [%s]",
              $sMyName, $sPsCmd, $iPid, $iExitCode );
            }
            else { # Success..
              &logTrace ( 20041,
              "<%s> Using search filter [%s] on CmdArgs [%s] PID [%s]",
              $sMyName, $sPattern, $sLine, $iPid );

              #-------------------------------
              # Check if we got what we want..
              #-------------------------------
              if ( $sLine =~ /$sPattern\b/o ) { # Found ns-slapd..
                &logTrace ( 20042,
                "<%s> +--- Found ns-slapd process with PID [%s] CmdArgs [%s] ---+",
                $sMyName, $iPid, $sLine );

                push ( @$rliOnlinePids, $iPid ); # Store..
                return $iReturnCode; # Get out..
              }
              else { # Oops! Stale PID!?
                &logWarn ( 20043,
                "<%s> Encountered stale PID [%s] with Args [%s] from PID file [%s]",
                $sMyName, $iPid, $sLine, $sPidFile );
              }
            }
          }
          else {
            &logTrace ( 20044,
            "<%s> PID [%s] from PID file [%s] does not exist",
            $sMyName, $iPid, $sPidFile );
          }
        }
        else {
          &logWarn ( 20045, "<%s> PID file [%s] seems corrupted",
          $sMyName, $sPidFile );
        }
      }
      else {
        &logWarn ( 20046, "<%s> Problems opening PID file [%s] for read: [%s]",
        $sMyName, $sPidFile, $! );
      }
    }
    else {
      &logTrace ( 20047, "<%s> Could not get any PID file for fast FLM",
      $sMyName );
    }

    &logTrace ( 20048,
    "<%s> Could not succeed with fast FLM. Attempting regular FLM..",
    $sMyName );
  }

  @$rliOnlinePids = (); # Reset..


  #----------------------------------
  # Set LANG for better performance..
  #----------------------------------
  $sLang = $ENV { LANG }; # Backup..
  $ENV { LANG } = 'C';    # Now set..

  #-----------------------------------
  # Call processes and store in hash..
  #-----------------------------------
  $iProcessCount = Proc::GetLongProcessListHash ( \@hProcListHash );

  #----------
  # Restore..
  #----------
  defined $sLang ? $ENV { LANG } = $sLang:delete $ENV { LANG }; # Restore..

  #----------------------
  # Check if we got any..
  #----------------------
  if ( 0 == $iProcessCount ) { # Oops!
    &logError ( 20049,
    "<%s> Call to get process list did not return any processes",
    $sMyName );
    return VCSDef_MonitorExit::OFFLINE; # Offline..
  }

  &logTrace ( 20050, "<%s>
  -----------------------------------------------
  Got [%s] processes from proc table. Using regex
  [%s] 
  for filtering
  -----------------------------------------------",
  $sMyName, scalar @hProcListHash, $sPattern );

  #-----------------------
  # Search for processes..
  #-----------------------
  foreach $rhlProc ( @hProcListHash ) {
    #--------------------------------------------------------------
    # Each $rhlProc is a ref. to a hash with the following keys..
    # "UID"     =>  <user id>
    # "PID"     =>  <process id>
    # "PPID"    =>  <parent process id>
    # "COMMAND" =>  <command string>
    #--------------------------------------------------------------

    next unless defined $$rhlProc { UID }; # Bug in Proc::GetLongProcessListHash..
    next if $$rhlProc { PID } == $$;       # Skip self..
    $iPid = $$rhlProc { PID };             # Store..

    &logTrace ( 20051, "<%s> Searching PID [%s] CMD [%s]",
    $sMyName, $iPid, $$rhlProc { COMMAND } );

    if ( $$rhlProc { COMMAND } =~ /$sPattern\b/o ) { # Found ns-httpd..
      &logTrace ( 20052,
      "<%s> +--- FOUND [ns-slapd] process with PID [%s] Cmdargs [%s] ---+",
      $sMyName, $iPid, $$rhlProc { COMMAND } );

      push ( @$rliOnlinePids, $iPid ); # Store..
    }
  }

  #------
  # Log..
  #------
  if ( scalar @$rliOnlinePids ) { # Located ns-slapd..
    &logTrace ( 20053, "<%s> Setting return code to [%s]",
    $sMyName, $iReturnCode );
  }
  else {
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
    &logTrace ( 20054, "<%s> LDAP not up. Setting return code to [%s]",
    $sMyName, $iReturnCode );
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int Monitor ( IN String InstanceRoot,
                          IN String LDAPHost,
                          IN String LDAPPort,
                          IN String MonitorProgram,
                          IN Integer SSLPort,
                          IN Integer SecondLevelMonitor,
                          IN String ResourceName );

 Description : Performs the in-depth monitoring

 Parameters :

 $sInstanceRoot             Absolute path to instance of Directory Server
 $sLDAPHost                 Hostname/IPv4 address of virtual host for Directory Server
 $iLDAPPort                 The port at which the Directory Server listens
 $sMonitorProgram           Full path to external monitor program
 $iSSLPort                  SSL Port on which the Directory Server listens
 $iSecondLevelMonitor       Cycle count to run in-depth monitoring
 $sResourceName             Name of this VCS resource

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub Monitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot,
       $sLDAPHost,
       $iLDAPPort,
       $sMonitorProgram,
       $iSSLPort,
       $iSecondLevelMonitor,
       $sResourceName ) = @_;

 #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20055, "Subroutine <%s> called:
  InstanceRoot           [%s]
  LDAPHost               [%s]
  LDAPPort               [%s]
  MonitorProgram         [%s]
  SSLPort                [%s]
  SecondLevelMonitor     [%s]
  ResourceName           [%s]",

  $sMyName,
  $sInstanceRoot, $sLDAPHost, $iLDAPPort, $sMonitorProgram,
  $iSSLPort, $iSecondLevelMonitor, $sResourceName );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  #-------------------------------------------------------
  # Upper bound..
  # Time in EPOCH when entry pt started + VCS time limit..
  #-------------------------------------------------------
  my $iTillWhen = $^T + SunJESLDAP::GetEngineTimeout ( 1 ); # With margin..

  #----------------------------------------------
  # In-depth monitoring, if correct time to run..
  #----------------------------------------------
  if ( VCSagentFW::RunSecondLevelMonitor ( $iSecondLevelMonitor ) ) {
    #--------------------------
    # Just In Time Validation..
    #--------------------------
    if ( VCSDef_MonitorExit::ONLINE == ( $iReturnCode =
         SunJESLDAP::DelayedAttributeCheck ( $sInstanceRoot,
                                             $sLDAPHost,
                                             $iLDAPPort,
                                             $iSSLPort ) ) ) {
      &logTrace ( 20056,
      "<%s> Delayed attribute check completed successfully",
      $sMyName );
    }
    else {
      &logError ( 20057,
      "<%s> Attribute validation failed. Returning [%s]",
      $sMyName, $iReturnCode );
      return $iReturnCode;
    }

    &logTrace ( 20058,
    "<%s> Proceeding with second level monitoring", $sMyName );

    if ( VCSDef_MonitorExit::ONLINE == ( $iReturnCode =
         SunJESLDAP::SecondLevelMonitor ( $sInstanceRoot,
                                          $sLDAPHost,
                                          $iLDAPPort,
                                          $iSSLPort,
                                          $iTillWhen ) ) ) {
    #----------------------------------
    # Online after in-depth monitoring..
    #----------------------------------
      &logTrace ( 20059,
      "<%s> Directory Server online with status [%s] " .
      "after second level monitoring",
      $sMyName, $iReturnCode );
    }
    else {
      &logError ( 20060,
      "<%s> Directory Server NOT online. Status [%s] " .
      "after second level monitoring",
      $sMyName, $iReturnCode );
      return $iReturnCode; # Get out..
    }
  }

  #------------------------------------------------
  # Exit if external monitor has not been defined..
  #------------------------------------------------
  return $iReturnCode unless defined $sMonitorProgram;

  #---------------------------------------
  # MonitorProgram defined..
  # Separate the executable and the args..
  #---------------------------------------
  my ( $sExtCmd, $sExtCmdArgs ) = $sMonitorProgram # Separate
                                =~ /(\S+)\s*(.*)/; # Cmd and args..
  $sExtCmdArgs = '' unless defined $sExtCmdArgs;   # Prevent undefs..

  if ( $sExtCmd ) {                                      # Defined..
    if ( VCSDef_MonitorExit::ONLINE == ( $iReturnCode =  # Check status..
         SunJESLDAP::ExternalMonitor ( $sExtCmd,         # After execution..
                                       $sExtCmdArgs,
                                       $iTillWhen ) ) ) {

      &logTrace ( 20061,
      "<%s> Directory server found online with status [%s]
      after executing external monitor [%s] with args [%s]",
      $sMyName, $iReturnCode, $sExtCmd, $sExtCmdArgs );
    }
    else {
      &logError ( 20062,
      "<%s> Directory Server NOT online with status [%s]
      after executing external monitor [%s] with args [%s]",
      $sMyName, $iReturnCode, $sExtCmd, $sExtCmdArgs );
    }
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : Int DelayedAttributeCheck ( IN String InstanceRoot,
                                        IN String LDAPHost,
                                        IN Integer LDAPPort,,
                                        IN Integer SSLPort );

 Description : Just In Time Validation

 Parameters :

 $sInstanceRoot   Absolute path to instance of Directory Server
 $sLDAPHost       Hostname/IPv4 address of virtual host for Directory Server
 $iLDAPPort       The port at which the Directory Server listens
 $iSSLPort        SSL Port on which the Directory Server listens

 Returns  : Integer value as defined below:

 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub DelayedAttributeCheck {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot,
       $sLDAPHost,
       $iLDAPPort,
       $iSSLPort ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20063, "Subroutine <%s> called:
  InstanceRoot     [%s]
  LDAPHost         [%s]
  LDAPPort         [%s]
  SSLPort          [%s]",

  $sMyName,
  $sInstanceRoot, $sLDAPHost, $iLDAPPort, $iSSLPort );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my @lsArgs = qw ( InstanceRoot LDAPHost LDAPPort SSLPort );
  my @lsReqdArgs = qw ( InstanceRoot LDAPHost LDAPPort SSLPort );
  my ( @lsMissing, %hsArgList ) = ();

  @hsArgList { @lsArgs } = @_; # Assign..

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20064,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #--------------------
  # Return, if failed..
  #--------------------
  if ( VCSDef_MonitorExit::ONLINE != $iReturnCode ) {
    return $iReturnCode;
  }

  #-----------------------------------------
  # Validate each of the attribute in turn..
  #-----------------------------------------

  #------------------------
  # ServerRoot must exist..
  #------------------------
  if ( ! -d $sInstanceRoot ) {
    &logError ( 20065,
    "<%s> InstanceRoot directory [%s] does not exist",
    $sMyName, $sInstanceRoot );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }

  #--------------------
  # Validate hostname..
  #--------------------
  if ( ! inet_aton ( $sLDAPHost ) ) {
    &logError ( 20066,
    "<%s> Could not resolve LDAPHost [%s]", $sMyName, $sLDAPHost );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }

  #------------
  # LDAP Port..
  #------------
  if ( $iLDAPPort <= 0 ) {
    &logError ( 20067, "<%s> LDAPPort [%s] has to be greater than 0",
    $sMyName, $iLDAPPort );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : Int SecondLevelMonitor ( IN String IntanceRoot
                                     IN String LDAPHost,
                                     IN Integer LDAPPort,
                                     IN Integer LDAPPort,
                                     IN Integer TillWhen );

 Description : In-depth monitoring, if enabled.

 Parameters :
 $sInstanceRoot   Absolute path to instance of Directory Server
 $sLDAPHost       Hostname/IPv4 address of virtual host for Directory Server
 $iLDAPPort       The port at which the Directory Server listens
 $iSSLPort        SSL Port on which the Directory Server listens
 $iTillWhen       Absolute time in EPOCH to engine timeout

 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN  State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub SecondLevelMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot,
       $sLDAPHost,
       $iLDAPPort,
       $iSSLPort,
       $iTillWhen ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20068, "Subroutine <%s> called:
  InstanceRoot      [%s] 
  LDAPHost          [%s]
  LDAPPort          [%s]
  SSLPort           [%s]
  TillWhen          [%s]",

  $sMyName,
  $sInstanceRoot, $sLDAPHost, $iLDAPPort,
  $iSSLPort, $iTillWhen );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sServerRoot, $sLibPath, @lsOldLibPath, $iTimeout, $iExitCode,
       @lsCmdOut, @lsLibVars ) = ();

  ( $sServerRoot = $sInstanceRoot ) =~ s#(.*)/.*$#$1#; # Dirname..
  my $sMonitorCmd = "$sServerRoot/shared/bin/ldapsearch";
  my $sMonitorCmdArgs = "-h $sLDAPHost:$iLDAPPort -b cn=monitor -s base " .
                        "objectclass=\* version";
  my $sSwitch = $^O; # Get current OS..
  
  #------------------
  # Validate binary..
  #------------------
  if ( ! -e $sMonitorCmd ) {
    &logError ( 20069, "<%s> Monitor utility [%s] does not exist",
    $sMyName, $sMonitorCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  elsif ( ! -x $sMonitorCmd ) {
    &logError ( 20070, "<%s> Monitor utility [%s] not executable",
    $sMyName, $sMonitorCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  else {
    &logTrace ( 20071, "<%s> Monitor utility [%s] located",
    $sMyName, $sMonitorCmd );
  }

  #----------------------------------
  # Exit, if not errors encountered..
  #----------------------------------
  unless ( $iReturnCode == VCSDef_MonitorExit::ONLINE ) {
    return $iReturnCode;
  }
  
  #----------------------------------
  # Implement a C look-alike switch..
  #----------------------------------
  #goto $sSwitch; {
  if ( "solaris" eq $sSwitch ) {
      push ( @lsLibVars, 'LD_LIBRARY_PATH' );
  }
  elsif ( "hpux" eq $sSwitch || "aix" eq $sSwitch ) {
      push ( @lsLibVars, 'LD_LIBRARY_PATH',
                         'SHLIB_PATH',
                         'LIBPATH' );
  }
  elsif ( "linux" eq $sSwitch ) {
      push ( @lsLibVars, 'LIBRARY_PATH' );
  }

  #--------------------
  # Get time bearings..
  #--------------------
  if ( ( $iTimeout = $iTillWhen - ( time + 5 ) ) <= 10 ) {
    &logError ( 20072,
    "<%s> Insufficient time to do in-depth monitoring. Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );
    return VCSDef_MonitorExit::UNKNOWN; # Error..
  }

  #-------------------
  # Save old environ..
  #-------------------
  foreach $sSwitch ( @lsLibVars ) {
    if ( $ENV { $sSwitch } ) {
      push ( @lsOldLibPath, $ENV { $sSwitch } );
      &logTrace ( 20073, "<%s> Env variable [%s] Old value [%s]",
      $sMyName, $sSwitch, $ENV { $sSwitch } );
    }
  }

  #-------------------
  # Set to new value..
  #-------------------
  @ENV { @lsLibVars } = "$sServerRoot/lib" x scalar @lsLibVars;

  #-----------------------
  # Display the new ones..
  #-----------------------
  foreach $sSwitch ( @lsLibVars ) {
    &logTrace ( 20074, "<%s> Env variable [%s] New value [%s]",
    $sMyName, $sSwitch, $ENV { $sSwitch } );
  }

  #----------------------
  # Execute the command..
  #----------------------
  if ( SunJESLDAP::ExecLocalCmd ( $sMonitorCmd,
                                  $sMonitorCmdArgs,
                                  \$iExitCode,
                                  $iTimeout,
                                  \@lsCmdOut ) ) {
    &logTrace ( 20075,
    "<%s> Command [%s] with args [%s] executed successfully with exit code [%s]",
    $sMyName, $sMonitorCmd, $sMonitorCmdArgs, $iExitCode );
  }
  elsif ( ! defined $iExitCode ) {
    $lsCmdOut [0] ||= 'undef'; # Set if not defined..

    if ( $lsCmdOut [0] eq 'timeout' ) { # Timed out..
      $iReturnCode = VCSDef_MonitorExit::TIMEOUT;

      &logError ( 20076,
      "<%s> Command [%s] with args [%s] timed out. Returning [%s]",
      $sMyName, $sMonitorCmd, $sMonitorCmdArgs, $iReturnCode );
    }
    else {
      $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
    }
  }
  else { # Failed..
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
    &logError ( 20077,
    "<%s> Command [%s] with args [%s] execution failed with exit code [%s].
    Cmd output [%s]. Returning [%s]",
    $sMyName, $sMonitorCmd, $sMonitorCmdArgs, $iExitCode,
    join ( chr 012, @lsCmdOut ), $iReturnCode );
  }

  #------------------------
  # Restore env variables..
  #------------------------
  if ( scalar @lsOldLibPath ) {          # If we saved any..
    delete @ENV { @lsLibVars };          # First delete all we defined..
    @ENV { @lsLibVars } = @lsOldLibPath; # Now restore..
  }

  unless ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    return $iReturnCode; # Get out..
  }

  #-------------------------------------
  # Check if SSLPort is up, if defined..
  #-------------------------------------
  if ( $iSSLPort ) {
    &logTrace ( 20078,
    "<%s> Checking server status on SSLPort [%s]", $sMyName, $iSSLPort );

    if ( Comms::CheckPort ( $sLDAPHost,
                            $iSSLPort ) ) {
      &logTrace ( 20079,
      "<%s> Server listening on Host [%s] Port [%s]: Socket method",
      $sMyName, $sLDAPHost, $iSSLPort );
    }
    else {
      &logError ( 20080,
      "<%s> Server NOT listening on Host [%s] Port [%s]: Socket method",
      $sMyName, $sLDAPHost, $iSSLPort );
      $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
    }
  }

  &logTrace ( 20081, "<%s> Returning with return code [%s]",
  $sMyName, $iReturnCode );

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool ExecLDAPCmd ( IN String InstanceRoot,
                               IN String SSLDbPasswd );

 Description : Starts/Stops a Sun ONE Directory Server

 Parameters :

 $sInstanceRoot  Absolute path to InstanceRoot of Admin Server
 $sSSLDbPasswd   Encrypted password for trusted database using
                 vcsencrypt

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ExecLDAPCmd {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot,
       $sSSLDbPasswd ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20082, "Subroutine <%s> called:
  InstanceRoot        [%s]
  SSLDbPasswd         [%s]",

  $sMyName, $sInstanceRoot, $sSSLDbPasswd );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = 1; # Success..

  #-------------------------
  # Define local variables..
  #-------------------------
  my %hLDAPCmd = ( 'SunJESLDAP::Start', 'start',
                   'SunJESLDAP::Stop',  'stop',
                   'SunJESLDAP::Kill',  'stop' ); 

  my ( $sCmd, $sCmdArgs, $sAction, $iExitCode, $bTimedOut,
       $iTillWhen, @lsCmdOut ) = ();

  my $sMyCaller = ( caller 1 )[3];   # Get calling function..
  my $sDecryptCmd = $sCmdVcsDecrypt; # Get global..

  #-------------------------------
  # Set appropriate LDAP command..
  #-------------------------------
  if ( $sAction = $hLDAPCmd { $sMyCaller } ) {
    $sCmd = "$sInstanceRoot/${sAction}-slapd"; 

    if ( ! -e $sCmd ) {
      &logError ( 20083, "<%s> Command [%s] to [%s] does not exist",
      $sMyName, $sCmd, $sAction );
      $iReturnCode = 0; # Failure..
    }
    elsif ( ! -x $sCmd ) {
      &logError ( 20084, "<%s> Command [%s] to [%s] not executable",
      $sMyName, $sCmd, $sAction );
      $iReturnCode = 0; # Failure..
    }
    else { 
      &logTrace ( 20085, "<%s> Using [%s] to [%s] the Directory Server",
      $sMyName, $sCmd, $sAction );
    }
  }
  else {
    &logError ( 20086, "<%s> Got unsupported call from sub-routine [%s]",
    $sMyName, $sMyCaller );
    $iReturnCode = 0; # Failure..
  }
 
  $iReturnCode || return $iReturnCode; # If failed, return..
 
  #------------------------------------
  # Protected database passwd defined..
  #------------------------------------
  if ( $sSSLDbPasswd =~ /\S/ && $sAction eq 'start' ) {
    if ( ! -e $sDecryptCmd ) {
      &logError ( 20087, "<%s> Decrypt utility [%s] does not exist",
      $sMyName, $sDecryptCmd );
      $iReturnCode = 0; # Failure..
    }
    elsif ( ! -x $sDecryptCmd ) {
      &logError ( 20088, "<%s> Decrypt utility [%s] not executable",
      $sMyName, $sDecryptCmd );
      $iReturnCode = 0; # Failure..
    }  

    $iReturnCode || return $iReturnCode; # If failed, return..
  
    &logTrace ( 20089, "<%s> Decrypt utility [%s] located",
    $sMyName, $sDecryptCmd );

    #----------------------------------------
    # Adjust cmd and cmd args appropriately..
    #----------------------------------------
    $sCmdArgs = "$sSSLDbPasswd | /bin/awk '{ print }' | $sCmd",
    $sCmd = "$sDecryptCmd";
  }

  $sCmdArgs = '' unless defined $sCmdArgs; # Prevent undefs..

  &logTrace ( 20090,
  "<%s> Executing command [%s] with args [%s] to [%s] the Directory Server",
  $sMyName, $sCmd, $sCmdArgs, $sAction );

  #--------------------
  # Get time bearings..
  #--------------------
  $iTillWhen = $^T + SunJESLDAP::GetEngineTimeout ( 1 ) - time; # With margin..

  #----------------------
  # Execute the command..
  #----------------------
  if ( $iReturnCode = SunJESLDAP::ExecLocalCmd ( $sCmd,
                                                 $sCmdArgs,
                                                 \$iExitCode,
                                                 $iTillWhen,
                                                 \@lsCmdOut ) ) {
    &logTrace ( 20091,
    "<%s> Command [%s] executed successfully with exit code [%s]",
    $sMyName, $sCmd, $iExitCode );
  }
  else { # Failed..
    &logError ( 20092,
    "<%s> Command [%s] execution failed with exit code [%s]. Cmd output [%s]",
    $sMyName, $sCmd, $iExitCode, join ( chr 012, @lsCmdOut ) );
    $iReturnCode = 0; # Failure..
  }
  
  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : list GetPidFiles ( IN String InstanceRoot );

 Description : Locates PID files for a Sun ONE Directory Server

 Parameters :

 $sInstanceRoot       Absolute path to InstanceRoot of LDAP Server

 Returns : List of PID files for the Directory Server
 -------------------------------------------------------------------------

=cut

sub GetPidFiles {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20093, "Subroutine <%s> called:
  InstanceRoot        [%s]",
  $sMyName, $sInstanceRoot );

  #----------------------
  # Set local variables..
  #----------------------
  my @lsPidFiles = ();
  my $sPidFile = "$sInstanceRoot/logs/pid";

  #-----------------
  # Test each file..
  #-----------------
  if ( ! -e $sPidFile ) {
    &logTrace ( 20094, "<%s> Could not locate PID file [%s]",
    $sMyName, $sPidFile );
  }
  elsif ( ! -s $sPidFile ) {
    &logWarn ( 20095, "<%s> PID file [%s] is of zero bytes",
    $sMyName, $sPidFile );
  }
  else {
    &logTrace ( 20096,
    "<%s> Located PID file [%s] for Directory Server",
    $sMyName, $sPidFile );
    push ( @lsPidFiles, $sPidFile );
  }

  return @lsPidFiles;
}


=pod

 -------------------------------------------------------------------------
 Function : int GetEngineTimeout ( IN Bool UseMargin );

 Description : Calculates engine timeout for entry point
 
 Parameters  :

 $bUseMargin      Flag to indicate if safety margin is requested

                  boolean value in the set of {0,1} where
                  0 No safety margin needed
                  1 Safety margin to be used
 
 Returns:  Time in secs.
 -------------------------------------------------------------------------

=cut

sub GetEngineTimeout {
  #----------------
  # Get Arguments..
  #----------------
  my ( $bUseMargin ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20097, "Subroutine <%s> called with:
  UseMargin  [%s]",
  $sMyName, $bUseMargin );

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sAttribute, $sCommand, $sCommandArgs,
       $iEngineTimeout, $iExitCode ) = ();

  #--------------------------------------------------
  # Attempt to get resource level overridden values..
  #--------------------------------------------------
  my $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };
  my ( $sEntryPointName, $sResourceType ) = reverse $0 =~ m@([^/]+)@g;
  $sResourceType = 'SunJESLDAP' if $sResourceType !~ /\w/; # Ensure..
  ( $sAttribute = $sEntryPointName ) =~ s/(.*)/\u$1Timeout/;

  #----------------------------------
  # Grace margins from VCS timeouts..
  #----------------------------------
  my %hiGraceMargin = ( 'MonitorTimeout', 10,   # Seconds..
                        'OnlineTimeout',  10,
                        'OfflineTimeout', 10,
                        'CleanTimeout',   20 );

  #-----------------------------------
  # Query if it has already been set..
  #-----------------------------------
  if ( defined $::giEPTimeout &&    # Defined..
       $::giEPTimeout =~ /^\d+/ ) { # to an integer..
    $iEngineTimeout = $::giEPTimeout;

    &logTrace ( 20098,
    "<%s> Using AEPTimeout [%s] as EP timeout value",
    $sMyName, $iEngineTimeout );
  }
  else {
    #-----------------------------------------
    # Form query string based on entry point..
    #-----------------------------------------
    $sCommand = $sCmdHares; # Get global..
    $sCommandArgs = "-value $sResourceName $sAttribute";
 
    #----------------------------------
    # Fire command and collect output..
    #----------------------------------
    ( $iEngineTimeout = readpipe ( "$sCommand $sCommandArgs 2>/dev/null" ) )
                      =~ s/\n//;
 
    #--------------------
    # Check exit status..
    #--------------------
    if ( $iExitCode = $? >> 8 ) {
      &logTrace ( 20099,
      "<%s> Command [%s] with Args [%s] failed with exitcode [%s].",
      $sMyName, $sCommand, $sCommandArgs, $iExitCode );
    }
    elsif ( ! $iEngineTimeout ) { # Undefined value..
      &logTrace ( 20100, "<%s> Could not get [%s] from engine",
      $sMyName, $sAttribute );
    }
    else { # Success..
      &logTrace ( 20101, "<%s> Got [%s] as [%s] sec from engine",
      $sMyName, $sAttribute, $iEngineTimeout );
    }

    #-----------------------------------------
    # Form query string based on entry point..
    #-----------------------------------------
    unless ( $iEngineTimeout ) {
      $sCommand = $sCmdHatype; # Get global..
      $sCommandArgs = "-value $sResourceType $sAttribute";
    
      #----------------------------------
      # Fire command and collect output..
      #----------------------------------
      ( $iEngineTimeout = readpipe ( "$sCommand $sCommandArgs" ) ) =~ s/\n//;
    
      #--------------------
      # Check exit status..
      #--------------------
      if ( $iExitCode = $? >> 8 ) {
        &logTrace ( 20102,
        "<%s> Command [%s] with Args [%s] failed with exitcode [%s]",
        $sMyName, $sCmdHatype, $sCommandArgs, $iExitCode );
        $iEngineTimeout = $bUseMargin ? $hiGraceMargin { $sAttribute } : 0;
      }
      elsif ( ! $iEngineTimeout ) { # Empty value..
        &logTrace ( 20103, "<%s> Could not get [%s] from engine",
        $sMyName, $sAttribute );
        $iEngineTimeout = $bUseMargin ? $hiGraceMargin { $sAttribute } : 0;
      }
      else { # Success..
        &logTrace ( 20104, "<%s> Got [%s] as [%s] sec from engine",
        $sMyName, $sAttribute, $iEngineTimeout );
      }
    }
    else {
     &logTrace ( 20105, "<%s> Got [%s] as [%s] sec from engine",
      $sMyName, $sAttribute, $iEngineTimeout );
    }
  }

  #----------------------------
  # Apply margin if requested.. 
  #----------------------------
  if ( $bUseMargin     &&                                    # Margin requested..
       $iEngineTimeout >= $hiGraceMargin { $sAttribute } ) { # And valid..
    $iEngineTimeout -= $hiGraceMargin { $sAttribute };       # Apply..

    &logTrace ( 20106,
    "<%s> Setting up [%s] sec margin. Returning [%s]",
    $sMyName, $hiGraceMargin { $sAttribute }, $iEngineTimeout );
  }

  return ( $iEngineTimeout ); # Secs..
}

=pod

 -------------------------------------------------------------------------
 Function : bool KillLDAPProcs ( IN String InstanceRoot );

 Description : Kills processes for a Sun ONE Directory Server

 Parameters :

 $sInstanceRoot       Absolute path to InstanceRoot of LDAP Server

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub KillLDAPProcs {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstanceRoot ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20107, "Subroutine <%s> called:
  InstanceRoot        [%s]",
  $sMyName, $sInstanceRoot );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..

  #----------------------
  # Set local variables..
  #----------------------
  my ( @liGoners, @liUnDead ) = ();

  #--------------------------
  # If called during online..
  #--------------------------
  if ( ( caller 1 )[3] eq 'SunJESLDAP::Start' ) {
    @liGoners = @lsRunningProcs; # Get global..
  }
  else {
    #----------------------------
    # Get processes for killing..
    #----------------------------
    SunJESLDAP::GetProcState ( $sInstanceRoot, \@liGoners, 0 );
  }

  #------------------------------------------------------------------
  # We do a two level kill. The SIGINQ ensures  that  the  PID  is  a
  # valid process before we actually issue the SIGKILL. We don't want
  # to report a failure for SIGKILL in case  of  defunct  or  missing
  # processes.
  #------------------------------------------------------------------
  if ( scalar @liGoners ) {
    #----------------------
    # Failed to kill some..
    #----------------------
    if ( @liUnDead = grep {
                         kill ( 0, $_ ) && # Valid..
                       ! kill ( 9, $_ )    # and could NOT be killed..
                     } @liGoners ) {       # From what's found..
      &logError ( 20108,
      "<%s> PIDs not killed [%s] Total not killed [%s]",
      $sMyName, join ( ' ', @liUnDead ), scalar @liUnDead );
      $iReturnCode = 0; # Failure..
    }
    else { # All killed..
      &logInfo ( 20109,
      "<%s> Killed PIDs [%s] Total killed: [%s]",
      $sMyName, join ( ' ', @liGoners ), scalar @liGoners );
    }
  }
  else {
    &logTrace ( 20110,
    "<%s> Could not locate any process for InstanceRoot [%s]",
    $sMyName, $sInstanceRoot );
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int ExternalMonitor ( IN String Cmd,
                                  IN String CmdArgs,
                                  IN Integer TillWhen );

 Description : Handles user defined external monitor program

 Parameters :

 $sCmd        Absolute path to external monitor program
 $sCmdArgs    Command line arguments, if any, for Cmd
 $iTillWhen   Represents the upper limit set by engine in EPOCH 

 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub ExternalMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sCmd,
       $sCmdArgs,
       $iTillWhen ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20111, "Subroutine <%s> called with:
  Cmd         [%s]
  CmdArgs     [%s]
  TillWhen    [%s]",

  $sMyName,
  $sCmd, $sCmdArgs, $iTillWhen );

  #------------------
  # Hopeful success..
  #------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #------------------------
  # Setup local variables..
  #------------------------
  my ( $iExitCode, $iTimeout, @lsCmdOut ) = ();

  #-------------------------
  # Handled exit code hash..
  #-------------------------
  my %hiExitCodes = (  0, VCSDef_MonitorExit::ONLINE,
                     110, VCSDef_MonitorExit::ONLINE,
                       1, VCSDef_MonitorExit::OFFLINE,
                     100, VCSDef_MonitorExit::OFFLINE,
                          VCSDef_MonitorExit::TIMEOUT,
                          VCSDef_MonitorExit::TIMEOUT );


  #---------------------------------------------
  # Check if command can be found and executed..
  #---------------------------------------------
  if ( ! -e $sCmd ) {
   &logError ( 20112,
    "<%s> External monitor [%s] does not exist",
    $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( ! -x $sCmd ) {
   &logError ( 20113,
    "<%s> External monitor [%s] not executable.",
    $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( ( $iTimeout = $iTillWhen - ( time + 5 ) ) <= 5 ) { # Lack of time..
   &logError ( 20114,
    "<%s> Insufficient time to proceed with external monitor [%s].
    Short by [%s] sec", $sMyName, $sCmd, abs ( $iTimeout ) );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;  
  }

  #--------------------------------------------
  # Return if validation failed at this point..
  #--------------------------------------------
  unless ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    return $iReturnCode;
  }

  &logTrace ( 20115,
  "<%s> Executing external monitor with Command [%s] Args " .
  "[%s] timeout [%s] secs",
  $sMyName, $sCmd, $sCmdArgs, $iTimeout );

  #----------------------
  # Execute the command..
  #----------------------
  SunJESLDAP::ExecLocalCmd ( $sCmd,
                             $sCmdArgs,
                             \$iExitCode,
                             $iTimeout,
                             \@lsCmdOut );

  #------------------------
  # Report on exit status..
  #------------------------
  if ( ! defined $iExitCode ) {
    $lsCmdOut [0] ||= 'undef'; # Set if not defined..

    if ( $lsCmdOut [0] eq 'timeout' ) {
      $iExitCode = VCSDef_MonitorExit::TIMEOUT;

      &logError ( 20116,
      "<%s> Command [%s] with args [%s] timed out. Returning [%s]",
      $sMyName, $sCmd, $sCmdArgs, $iExitCode );
    }
    else {
      $iExitCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
    }
  }
  elsif ( $hiExitCodes { $iExitCode } &&
       VCSDef_MonitorExit::ONLINE ==
       $hiExitCodes { $iExitCode } ) {
    &logTrace ( 20117,
    "<%s> Command [%s] with args [%s] executed successfully with exit code [%s].
    Cmd output [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iExitCode, join ( chr 012, @lsCmdOut ) );
  }
  else {
    &logWarn ( 20118,
    "<%s> Command [%s] with args [%s] execution failed with exit code [%s].
    Cmd output [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iExitCode, join ( chr 012, @lsCmdOut ) );
  }

  #--------------------------------
  # Setup appropriate return code..
  #--------------------------------
  $iReturnCode = $hiExitCodes { $iExitCode } ||
                 VCSDef_MonitorExit::UNKNOWN;

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool ExecLocalCmd ( IN String Command,
                                IN String CommandArgs,
                                OUT RefInt ExitCode,
                                IN Integer Timeout,
                                OUT ListReference CmdOut );

 Description : Wrapper function for
               Sys::RunWithEnvCmdWithOutputWithTimeOut to collect output
               of the command.
 
 Parameters :

 $sCommand         Absolute path to location of command
 $sCommandArgs     Arguments to command
 $riExitCode       Reference to collect exit code of command
 $iTimeout         Time limit to execute the command
 $rlsCmdOut        Stores o/p from command if successful

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ExecLocalCmd {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sCommand,
       $sCommandArgs,
       $riExitCode,
       $iTimeout,
       $rlsCmdOut ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20119, "Subroutine <%s> called with:
  Command     [%s]
  CommandArgs [%s]
  riExitCode  [%s]
  Timeout     [%s]
  rlsCmdOut   [%s]",

  $sMyName,
  $sCommand, $sCommandArgs, $riExitCode, $iTimeout, $rlsCmdOut );

  #---------------------------------------
  # Setup return code to hopeful success..
  #---------------------------------------
  my $iReturnCode = 1;

  #-----------------------------------
  # Verify timeout is greater than 1..
  #-----------------------------------
  if ( $iTimeout <= 1 ) {
    &logError ( 20120,
    "<%s> Insufficient time [%s] to execute Cmd [%s] with Args [%s].",
    $sMyName, $iTimeout, $sCommand, $sCommandArgs );
    return 0;
  }

  #-------------------
  # Set up arguments..
  #-------------------
  my $bSourceUserEnv = 0;   # Do not source..
  my $sEnvFile = "";        # No env file..
  my $sUser = 'root';       # Run as root..
  my $sTmpFile = tmpnam (); # Outfile..
  my $bTimedOut = 0;        # Timeout variable..

  @$rlsCmdOut = (); # Reset..

  $sCommandArgs .= " 1> $sTmpFile"; # Append..

  &logTrace ( 20121,
  "<%s> Executing Command [%s] with Args [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut ( $sEnvFile,
                                                           $bSourceUserEnv,
                                                           $sCommand,
                                                           $sCommandArgs,
                                                           $riExitCode,
                                                           $sUser,
                                                           $rlsCmdOut,
                                                           \$bTimedOut,
                                                           $iTimeout );
  #--------------------
  # Check return code..
  #--------------------
  if ( $iReturnCode ) {   # No error with command execution..
    if ( $$riExitCode ) { # Failure..
      &logTrace ( 20122,
      "<%s> Command [%s] with args [%s] returned exit code [%s]",
      $sMyName, $sCommand, $sCommandArgs, $$riExitCode );
      $iReturnCode = 0; # Failure..
    }
    else {
      &logTrace ( 20123,
      "<%s> Command [%s] with args [%s] executed successfully with exit code [%s]",
      $sMyName, $sCommand, $sCommandArgs, $$riExitCode );
    }

    #-------------------------
    # Now collect the output..
    #-------------------------
    if ( -s $sTmpFile ) {
      if ( open ( LOGFILE, $sTmpFile ) ) {
        @$rlsCmdOut = grep { s/\n// || 1 } <LOGFILE>; # Read file..

        &logTrace ( 20124, "<%s> Command output :----------:\n[%s]",
        $sMyName, join ( chr 012, @$rlsCmdOut ) );

        close ( LOGFILE ) ||
        &logWarn ( 20125, "<%s> Could not close [%s] after read: [%s]",
        $sMyName, $sTmpFile, $! );
      }
      else {
        &logWarn ( 20126, "<%s> Could not open [%s] for logging: [$!]",
        $sMyName, $sTmpFile, $! );
      }
    }
    else {
      &logTrace ( 20127,
      "<%s> Could not collect output of command", $sMyName );
    }
  }
  else { # Execution error..
    if ( $bTimedOut ) {  # We timed out..
      &logError ( 20128,
      "<%s> Command [%s] with args [%s] did not complete within [%s] seconds",
      $sMyName, $sCommand, $sCommandArgs, $iTimeout );
      $$rlsCmdOut [0] = 'timeout'; # Timeout..
    }
    else { # Failure due to something else..
      &logError ( 20129,
      "<%s> Failed in call to RunWithEnvCmdWithOutputWithTimeOut",
      $sMyName );
      @$rlsCmdOut = (); # Reset..
    }
  }

  #-----------
  # Clean up..
  #-----------
  unlink ( $sTmpFile ) ||
  &logWarn ( 20130, "<%s> Could not remove [%s] after read: [%s]",
  $sMyName, $sTmpFile, $! );

  return $iReturnCode;
}

1; # So PERL knows its included..
